package com.zeocollect;

import android.util.Log;

import androidx.annotation.NonNull;

import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.module.annotations.ReactModule;
import com.zeotap.collect.Collect;
import com.zeotap.collect.CollectOptions;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;

@ReactModule(name = ZeoCollectModule.NAME)
public class ZeoCollectModule extends ReactContextBaseJavaModule {
    public static final String NAME = "ZeoCollect";
    CollectOptions.Builder builder;
    Collect collect;
    ReactApplicationContext mReactContext;

    public ZeoCollectModule(ReactApplicationContext reactContext) {
        super(reactContext);
        mReactContext = reactContext;
    }

    @ReactMethod
    public void initalizeBySettingOptions(ReadableMap rawOptions) {
        try {
            builder = CollectOptions.builder(mReactContext);
            HashMap<String, Object> clientOptions = rawOptions.toHashMap();
            Object androidWriteKey = clientOptions.get("android_write_key");
            try {
                if (androidWriteKey == null || Objects.equals(androidWriteKey, "")
                        || !(androidWriteKey instanceof String)) {
                    throw new IllegalArgumentException("Invalid android write key.");
                }
            } catch (IllegalArgumentException e) {
                System.out.println(e.getMessage());
                return;
            }

            builder = builder.credential((String) androidWriteKey);

            Object logging = clientOptions.get("logging");
            if (logging != null && logging instanceof Boolean) {
                builder = builder.enableLogging((Boolean) logging);
            }
            Object batchSize = clientOptions.get("batch_size");
            if (batchSize != null && !Objects.equals(batchSize, "")
                    && (batchSize instanceof Integer || batchSize instanceof Double)) {
                if (batchSize instanceof Double) {
                    builder = builder.uploadBatchSize(((Double) batchSize).intValue());
                } else if (batchSize instanceof Integer) {
                    builder = builder.uploadBatchSize((Integer) batchSize);
                }
            }
            Object optOut = clientOptions.get("opt_out");
            if (optOut != null && optOut instanceof Boolean) {
                builder = builder.optOut((Boolean) optOut);
            }
            Object serviceInterval = clientOptions.get("service_interval");
            if (serviceInterval != null && !Objects.equals(serviceInterval, "")
                    && (serviceInterval instanceof Integer || serviceInterval instanceof Double)) {
                if (serviceInterval instanceof Double) {
                    builder = builder.serviceUploadInterval(((Double) serviceInterval).intValue());
                } else if (serviceInterval instanceof Integer) {
                    builder = builder.serviceUploadInterval((Integer) serviceInterval);
                }
            }
            Object useConsent = clientOptions.get("use_consent");
            if (useConsent != null && useConsent instanceof Boolean) {
                builder = builder.useConsent((Boolean) useConsent);
            }
            Object maxCacheSize = clientOptions.get("max_cache_size");
            if (maxCacheSize != null && !Objects.equals(maxCacheSize, "") &&
                    (maxCacheSize instanceof Integer || maxCacheSize instanceof Double)) {
                if(maxCacheSize instanceof Double){
                    builder = builder.maxCacheSize(((Double) maxCacheSize).intValue());
                } else if (maxCacheSize instanceof Integer){
                    builder = builder.maxCacheSize((Integer) maxCacheSize);
                }
            }
            Object userCountry = clientOptions.get("user_country");
            if (userCountry != null && !Objects.equals(userCountry, "") && userCountry instanceof String) {
                builder = builder.userCountry((String) userCountry);
            }
            Object checkForCmp = clientOptions.get("check_for_cmp");
            if (checkForCmp!= null && checkForCmp instanceof Boolean) {
                builder = builder.checkForCMP((Boolean) checkForCmp);
            }
            Object purposesForTracking = clientOptions.get("purposes_for_tracking");
            if (purposesForTracking != null && purposesForTracking instanceof List) {
                builder = builder.purposesForTracking(convertDoubleListToIntegerList((ArrayList<Object>) purposesForTracking));
            }
            Object purposesForIdentify = clientOptions.get("purposes_for_identify");
            if (purposesForIdentify != null && purposesForIdentify instanceof List) {
                builder = builder.purposesForIdentifying(convertDoubleListToIntegerList((ArrayList<Object>) purposesForIdentify));
            }
            Object areIdentitiesHashed = clientOptions.get("are_identities_hashed");
            Object hashIdentities = clientOptions.get("hash_identities");
            if (areIdentitiesHashed != null && areIdentitiesHashed instanceof Boolean) {
                builder = builder.areIdentitiesHashed((Boolean) areIdentitiesHashed);
                if (hashIdentities != null && hashIdentities instanceof Boolean) {
                    builder = builder.hashIdentities((Boolean) hashIdentities);
                }
            }
            Object checkZeotapVendorConsent = clientOptions.get("check_zeotap_vendor_consent");
            if (checkZeotapVendorConsent != null && checkZeotapVendorConsent instanceof Boolean) {
                builder = builder.checkZeotapVendorConsent((Boolean) checkZeotapVendorConsent);
            }
            CollectOptions options = builder.build();
            Collect.init(options);
            collect = Collect.getInstance();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private ArrayList<Integer> convertDoubleListToIntegerList(ArrayList<Object> list) {
        ArrayList<Integer> convertedList = new ArrayList<>();
        try {
            for (int i = 0; i < list.size(); i++) {
                if (list.get(i) instanceof Double) {
                    convertedList.add(((Double) list.get(i)).intValue());
                } else if (list.get(i) instanceof Integer) {
                    convertedList.add((Integer) (list.get(i)));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return convertedList;
    }

    @Override
    @NonNull
    public String getName() {
        return NAME;
    }

    @ReactMethod
    public void setEventNameProperties(String eventName) {
        if (collect != null) {
            collect.setEventProperties(eventName);
        }
    }

    @ReactMethod
    public void setEventProperties(String eventName, ReadableMap eventProp) {
        if (collect != null) {
            collect.setEventProperties(eventName, eventProp.toHashMap());
        }
    }

    @ReactMethod
    public void setInstantEventNameProperties(String eventName) {
        if (collect != null) {
            collect.setInstantEventProperties(eventName);
        }
    }

    @ReactMethod
    public void setInstantEventProperties(String eventName, ReadableMap eventProp) {
        if (collect != null) {
            collect.setInstantEventProperties(eventName, eventProp.toHashMap());
        }
    }

    @ReactMethod
    public void setUserIdentities(ReadableMap userProp) {
        if (collect != null) {
            collect.setUserIdentities(userProp.toHashMap());
        }
    }

    @ReactMethod
    public void setUserProperties(ReadableMap userProp) {
        if (collect != null) {
            collect.setUserProperties(userProp.toHashMap());
        }
    }

    @ReactMethod
    public void setPageProperties(ReadableMap properties) {
        if (collect != null) {
            collect.setPageProperties(properties.toHashMap());
        }
    }

    @ReactMethod
    public void unsetUserIdentities() {
        if(collect != null) {
            collect.unsetUserIdentities();
        }
    }

    @ReactMethod
    public void pauseCollection() {
        if(collect != null) {
            collect.pauseCollection();
        }
    }

    @ReactMethod
    public void resumeCollection() {
        if(collect != null) {
            collect.resumeCollection();
        }
    }

    @ReactMethod
    public void setConsent(ReadableMap consent) {
        if(collect != null) {
            collect.setConsent(consent.toHashMap());
        }
    }

    @ReactMethod
    public void listenToAskForConsent(Callback cb) {
        if(collect != null) {
            collect.listenToAskForConsent((data) -> cb.invoke());
        }
    }

    @ReactMethod
    public void getZI(Callback cb) {
        if(collect != null) {
            String zi = collect.getZI();
            cb.invoke(zi);
        }
    }

    @ReactMethod
    public void resetZI() {
        if(collect != null) {
            collect.resetZI();
        }
    }

    @ReactMethod
    public void setZI(String zi) {
        if(collect != null) {
            collect.setZI(zi);
        }
    }

    @ReactMethod
    public void setATTStatusAndAdvertisingId(String status, String advertisingId) {
        // do nothing
    }

    @ReactMethod
    public void setAdvertisingIdByATTStatus(String status) {
        // do nothing
    }

}
