import ZeotapCollect
import AppTrackingTransparency

@objc(ZeoCollect)
class ZeoCollect: NSObject {
    
    @objc func initalizeBySettingOptions(_ rawOptions: [String: NSObject]){
        do {
                var collectOptions = CollectOption()
                
                let writeKey = rawOptions["ios_write_key"]
                
                do {
                    guard let writeKey = writeKey as? String, !writeKey.isEmpty else {
                        throw NSError(domain: "ZeoCollect", code: 1, userInfo: [NSLocalizedDescriptionKey: "Invalid iOS write key."])
                    }
                } catch let error as NSError {
                        print(error.localizedDescription)
                        return
                }

                collectOptions = collectOptions.writeKey(value: writeKey as! String)
                
                if let logging = rawOptions["logging"] {
                    if logging is Bool {
                        collectOptions = collectOptions.logging(value: logging as! Bool)
                    }
                }
                if let batchSize = rawOptions["batch_size"] {
                    if batchSize is NSNumber {
                        collectOptions = collectOptions.batchSize(value: (batchSize as! NSNumber).intValue)
                    }
                }
                if let optout = rawOptions["opt_out"] {
                    if optout is Bool {
                        collectOptions = collectOptions.optout(value: optout as! Bool)
                    }
                }
                if let serviceInterval = rawOptions["service_interval"] {
                    if serviceInterval is NSNumber {
                        collectOptions = collectOptions.eventUploadInterval(value:(serviceInterval as! NSNumber).intValue)
                    }
                }
                if let useConsent = rawOptions["use_consent"] {
                    if useConsent is Bool {
                        collectOptions = collectOptions.useConsent(value: useConsent as! Bool)
                    }
                }
                if let maxCacheSize = rawOptions["max_cache_size"] {
                    if maxCacheSize is NSNumber {
                        collectOptions = collectOptions.maxCacheSize(value:(maxCacheSize as! NSNumber).intValue)
                    }
                }
                if let userCountry = rawOptions["user_country"] {
                    if userCountry is String {
                        collectOptions = collectOptions.userCountry(code: userCountry as! String)
                    }
                }
                if let checkForCMP = rawOptions["check_for_cmp"] {
                    if checkForCMP is Bool {
                        collectOptions = collectOptions.checkForCMP(value: checkForCMP as! Bool)
                    }
                }
                if let purposesForTracking = rawOptions["purposes_for_tracking"] {
                    if purposesForTracking is NSArray {
                        collectOptions = collectOptions.purposesForTracking(value: (purposesForTracking as! [Int]))
                    }
                }
                if let purposesForIdentify = rawOptions["purposes_for_identify"] {
                    if purposesForIdentify is NSArray {
                        collectOptions = collectOptions.purposesForIdentifying(value: (purposesForIdentify as! [Int]))
                    }
                }
                if let areIdentitiesHashed = rawOptions["are_identities_hashed"] {
                    if areIdentitiesHashed is Bool {
                        collectOptions = collectOptions.areIdentitiesHashed(value: areIdentitiesHashed as! Bool)
                    }
                    if let hashIdentities = rawOptions["hash_identities"] {
                        if hashIdentities is Bool {
                            collectOptions = collectOptions.hashIdentities(value: hashIdentities as! Bool)
                        }
                    }
                }
                if let checkZeotapVendorConsent = rawOptions["check_zeotap_vendor_consent"] {
                    if checkZeotapVendorConsent is Bool {
                        collectOptions = collectOptions.checkZeotapVendorConsent(value: checkZeotapVendorConsent as! Bool)
                    }
                }
                let collectOption = collectOptions.build()
                Collect.initialize(option: collectOption)

        } catch let err {
            Log.w(err.localizedDescription)
        }

    }
    
    @objc func setEventProperties(_ eventName: String, event eventProp: [String: NSObject]) {
      let prop = eventProp as! Dictionary<String, NSObject>
      if !prop.isEmpty {
        Collect.getInstance()?.setEventProperties(eventName, prop)
      }
    }
    
    @objc func setEventNameProperties(_ eventName: String) {
      Collect.getInstance()?.setEventProperties(eventName)
    }
    
    @objc func setInstantEventNameProperties(_ eventName: String) {
      Collect.getInstance()?.setInstantEventProperties(eventName)
    }
    
    @objc func setInstantEventProperties(_ eventName: String, event eventProp: [String: NSObject]) {
      Collect.getInstance()?.setInstantEventProperties(eventName, eventProp)
    }
    
    @objc func setUserProperties(_ user: [String: NSObject]) {
      Collect.getInstance()?.setUserProperties(user)
    }
    
    @objc func setPageProperties(_ page: [String: NSObject]) {
      Collect.getInstance()?.setPageProperties(page)
    }
    
    @objc func setUserIdentities(_ user: [String: NSObject]) {
      Collect.getInstance()?.setUserIdentities(user)
    }
    
    @objc func setConsent(_ consent: [String: NSObject]) {
      Collect.getInstance()?.setConsent(consent: consent)
    }
    
    @objc func listenToAskForConsent(_ cb: @escaping RCTResponseSenderBlock) {
      Collect.getInstance()?.listenToAskForConsent(action: { _data in
        cb([])
      })
    }
    
    @objc func unsetUserIdentities() {
      Collect.getInstance()?.unsetUserIdentities()
    }
    
    @objc func pauseCollection() {
      Collect.getInstance()?.pauseCollection()
    }
    
    @objc func resumeCollection() {
      Collect.getInstance()?.resumeCollection()
    }
    
    @objc func getZI(_ callback: @escaping RCTResponseSenderBlock) {
      let zi = Collect.getInstance()?.getZI()
      callback([zi ?? ""])
    }
    
    @objc func resetZI() {
      Collect.getInstance()?.resetZI()
    }
    
    @objc func setZI(_ zi: String) {
      Collect.getInstance()?.setZI(zi: zi)
    }
    
    
    @objc func setATTStatusAndAdvertisingId(_ status: String, _ advertisingId: String = "") {
        if #available(iOS 14, *) {
            Collect.getInstance()?.setATTStatusAndAdvertisingId(status: getAuthorizationStatus(status), advertisingId: advertisingId)
        } else {
            // Fallback on earlier versions
        }
    }

    @objc func setAdvertisingIdByATTStatus(_ status: String) {
        if #available(iOS 14, *) {
            Collect.getInstance()?.setAdvertisingIdByATTStatus(status: getAuthorizationStatus(status))
        } else {
            // Fallback on earlier versions
        }
    }

    @available(iOS 14, *)
    func getAuthorizationStatus(_ status: String) -> ATTrackingManager.AuthorizationStatus {
        switch status {
        case "authorized":
            return .authorized
        case "denied":
            return .denied
        case "restricted":
            return .restricted
        case "not-determined":
            return .notDetermined
        default:
            return .notDetermined
        }
    }
    
}
